% This script reproduces the results in Figure 5.2 in [1]
%
% [1] I. Dassios, K. Fountoulakis and J. Gondzio, "A Second-Order Method 
%     for Compressed Sensing Problems with Coherent and Redundant 
%     Dictionaries". Technical Report ERGO-14-007.
%
% The problem instance that is used has been copied from 
% version 1.2 of TFOCS package, url: http://cvxr.com/tfocs/.
%
% ** Signal Processing Toolbox is required **
%
% This file should take about 334 sec. CPU time to be completed.

clear;
tt = cputime;

randn('state',245);
rand('state',245);

% Generate phantom image.
psize = 256;

n1 = psize; 
n2 = n1;
N = n1*n2;
n = max(n1,n2);

x = phantom(n);

x = x(1:n1,1:n2);
x_original = x;
mat = @(x) reshape(x,n1,n2);
vec = @(x) x(:);

% Partial Fourier measurements.
M = round( N / 4 );
omega = randn(N,1);
[temp,omega] = sort(omega);
omega = sort( omega(1:M) );
downsample = @(x) x(omega);
SS.type = '()'; SS.subs{1} = omega; SS.subs{2} = ':';
upsample = @(x) subsasgn( zeros(N,size(x,2)),SS,x);

% take partial 2D DCT measurements.  We have Af(At) = identity (not
% vice-versa of course). Randomly permuted.
rp = randperm(N);
[~,rp_inv] = sort(rp);
rpF = @(x) x(rp);
rp_invF = @(x) x(rp_inv);
        
my_dct2 = @(x) dct(dct(x).').';
my_idct2= @(x) idct(idct(x).').';
Af = @(x) downsample(vec( my_dct2( mat(rpF(mat(x) )) ) ) );
At = @(y) vec( rp_invF( my_idct2( mat( upsample( y ) ) ) ) );


% Make matrix A.
A = linop_handles([M,N], Af, At );

% Noise function.
myAwgn = @(x,snr) x + ...
        10^( (10*log10(sum(abs(x(:)).^2)/length(x(:))) - snr)/20 )*randn(size(x));
    
% Generate samples.
b_original = Af(vec(x_original));
snr = 10;  % SNR in dB
x_noisy = myAwgn( x_original, snr );
b   = Af( vec(x_noisy) );

x_ref   = x_original;

sz              = A([],0);
M               = sz{2}(1);
N               = sz{1}(1);
[n1,n2]         = size(x_original);

fprintf('\tA is %s, %d x %d, signal has SNR %d dB\n', 'partial Fourier', M, N, round(snr) );

%% Call pdNCG solver for Figure 5.2.
Apdncg  = A_operator(@(w) A(w,1), @(w) A(w,2));

opts.n1  = n;
opts.n2  = n;
opts.b   = b;
opts.A   = Apdncg;

opts.tau = 0.0539; 
opts.mu = 1.0e-5; 
opts.cont_steps = 1; 
opts.tol = 1.0e-1; 
opts.prec_tol = 1.0e-15; % disable preconditioner
opts.problem  = 'tv';
opts.verbose  = 2;

fprintf('pdNCG is called with disabled preconditioning \n');
[~, ~, out] = pdNCGinterface(opts);
fprintf('CPU time for unpreconditioned case: %d \n',out.cput);

opts.prec_tol = 1.0e-1; % enable preconditioner for all iterations
opts.which_prec = 0;
        
fprintf('\n \n pdNCG is called with approximate preconditioning \n');
[~, ~, out] = pdNCGinterface(opts);
fprintf('CPU time for approximate preconditioner case: %d \n',out.cput);

fprintf('Elapsed CPU time for this file: %d sec. \n', cputime - tt);
clear;

