% This script reproduces Figure 7.2 in [1]
%
% [1] I. Dassios, K. Fountoulakis and J. Gondzio, "A Second-Order Method 
%     for Compressed Sensing Problems with Coherent and Redundant 
%     Dictionaries". Technical Report ERGO-14-007.
%
% The problem instance that is used has been copied from 
% TFOCS_paperExamplesOnly package, url: http://cvxr.com/tfocs/download/.
%
% ** Signal Processing Toolbox is required **
%
% This file should take about 80 sec. CPU time to be completed.

clear;
tt_file = cputime;

randn('state',245);
rand('state',245);

% Generate phantom image
psize = 256;

n1 = psize; 
n2 = n1;
N = n1*n2;
n = max(n1,n2);

% Select phantom image
x = phantom(n);

x = x(1:n1,1:n2);
x_original = x;
mat = @(x) reshape(x,n1,n2);
vec = @(x) x(:);

% Partial Fourier measurements.
M = round( N / 4 );
omega = randn(N,1);
[temp,omega] = sort(omega);
omega = sort( omega(1:M) );
downsample = @(x) x(omega);
SS.type = '()'; SS.subs{1} = omega; SS.subs{2} = ':';
upsample = @(x) subsasgn( zeros(N,size(x,2)),SS,x);

% take partial 2D DCT measurements.  We have Af(At) = identity (not
% vice-versa of course). Randomly permuted.
rp = randperm(N);
[~,rp_inv] = sort(rp);
rpF = @(x) x(rp);
rp_invF = @(x) x(rp_inv);
        
my_dct2 = @(x) dct(dct(x).').';
my_idct2= @(x) idct(idct(x).').';
Af = @(x) downsample(vec( my_dct2( mat(rpF(mat(x) )) ) ) );
At = @(y) vec( rp_invF( my_idct2( mat( upsample( y ) ) ) ) );

% treat X as n1*n2 x 1 vector?
VECTOR = true;
% or treat X as n1 x n2 matrix?
% VECTOR = false;

if VECTOR
    A = linop_handles([M,N], Af, At );
    VEC = vec;
else
    Af = @(x) vec(Af(x));
    At = @(x) mat(At(x));
    A = linop_handles({[n1,n2],[M,1]}, Af, At );
    VEC = @(x) x;
end
    
myAwgn = @(x,snr) x + ...
        10^( (10*log10(sum(abs(x(:)).^2)/length(x(:))) - snr)/20 )*randn(size(x));
    
b_original = Af(vec(x_original));
snr = 10;  % SNR in dB
x_noisy = myAwgn( x_original, snr );
b   = Af( vec(x_noisy) );
EPS = norm(b-b_original);

mu  = 1e-5*norm( linop_TV(x_original) ,Inf);

x_ref   = x_original;

%% Print Phantom clean.
figure(1)
imshow( x_original );
maxI    = 1; % max pixel value
PSNR    = @(x) 20*log10(maxI*sqrt(N)/norm(vec(x)-vec(x_original) ) );
title(sprintf('Phantom noiseless, PSNR is %.1f dB',...
    PSNR( x_original )));

%% Print Phantom noisy.
figure(2)
imshow( x_noisy );
maxI    = 1; % max pixel value
PSNR    = @(x) 20*log10(maxI*sqrt(N)/norm(vec(x)-vec(x_original) ) );
title(sprintf('Phantom noisy, PSNR is %.1f dB',...
    PSNR( x_noisy )));

sz              = A([],0);
M               = sz{2}(1);
N               = sz{1}(1);
[n1,n2]         = size(x_original);
norm_x_ref      = norm(x_ref,'fro');
er_ref          = @(x) norm(vec(x)-vec(x_ref))/norm_x_ref;

fprintf('\tA is %s, %d x %d, signal has SNR %d dB\n', 'partial Fourier', M, N, round(snr) );

%% Call TFOCS solver.
er              = er_ref;  
opts = [];
opts.restart    = 1000;
opts.errFcn     = @(f,dual,primal) er(primal);
opts.maxIts     = 250;

x0 = At(b);
z0  = [];   % we don't have a good guess for the dual
if VECTOR
    W   = linop_TV( [n1,n2] );
else
    W   = linop_TV( {n1,n2} );
end
normW           = linop_TV( [n1,n2], [], 'norm' );
opts.normW2     = normW^2;

tstart = cputime;
% [ x, out, optsOut ] = solver_sBPDN_W( A, W, b, EPS, mu, VEC(x0), z0, opts
% );
solver = @(mu,x0,z0,opts) solver_sBPDN_W( A, W, b, EPS, mu, x0, z0, opts);
[ x, out, optsOut ] = continuation(solver,5*mu,x0(:),z0,opts);
time_TFOCS = cputime - tstart;
fprintf('\n \n');

%% Print TFOCS
fprintf('TFOCS, Solution has %d nonzeros.  Error vs. original solution is %.2e\n',...
    nnz(x), er(x) );
x = mat(x);
figure(3)
imshow( x );
maxI    = 1; % max pixel value
PSNR    = @(x) 20*log10(maxI*sqrt(N)/norm(vec(x)-vec(x_original) ) );
title(sprintf('TFOCS, PSNR is %.1f dB, CPU time: %.1f sec.',...
    PSNR( x ), time_TFOCS));

%% Call pdNCG solver.
ApdNCG  = A_operator(@(w) A(w,1), @(w) A(w,2));

optspdNCG.n1  = n;
optspdNCG.n2  = n;
optspdNCG.b   = b;
optspdNCG.A   = ApdNCG;
optspdNCG.tau = 10/norm(out.dual{1});
optspdNCG.tol = 1.0e-1;
optspdNCG.problem  = 'tv';
optspdNCG.verbose  = 2;

tstart = cputime;
[xDC, dual, out] = pdNCGinterface(optspdNCG);
time_pdNCG = cputime - tstart;

%% Print pdNCG
fprintf('pdNCG, solution has %d nonzeros.  Error vs. original solution is %.2e\n',...
    nnz(xDC), er(xDC) );
xDC = mat(real(xDC));
figure(4)
imshow( [xDC] );
maxI    = 1; % max pixel value
PSNR    = @(x) 20*log10(maxI*sqrt(N)/norm(vec(x)-vec(x_original) ) );
title(sprintf('pdNCG, PSNR is %.1f dB, CPU time is %.1f',...
    PSNR( xDC ), time_pdNCG));

%disp(norm(xDC(:) - x(:))/norm(xDC(:)));

fprintf('Elapsed CPU time for this file: %d sec. \n', cputime - tt_file);
clear all;

