% This script reproduces Figure 7.3 in [1]
%
% [1] I. Dassios, K. Fountoulakis and J. Gondzio, "A Second-Order Method 
%     for Compressed Sensing Problems with Coherent and Redundant 
%     Dictionaries". Technical Report ERGO-14-007.
%
% The problems which are run in the file have been obtained from the
% problem set in http://dsp.rice.edu/cscamera.
%
% This file should take about 351 sec. CPU time to be completed.

clear;
tt_file = cputime;
%
% SCRIPT
%

% Picture size (sidelength in pixels). Options include 32, 64, 128
sidelength = 64;
% Subsampling rate: ratio between M (number of samples) and image size in
% pixels (picsize^2)
subrate = 0.4;

avg_cpu_tfocs = 0;
avg_cpu_pdncg = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% HERE FOR DICE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test picture name. Options include 'Ball','Mug','Dice','Logo', 'R'
picname = 'Dice';

% Load measurements - Column Vector
eval(['load ' picname '_' num2str(sidelength)]);

% Image size
N = sidelength^2;

% Get desired number of samples
y = y(1:(round(N*subrate)));

% Number of measurements
M = length(y);

% Load measurement matrix and truncate
% Each row of Phi contains a measurement vector of length N
% and is a raster scan of the 0/1 pattern fed to the DMD
eval(['load Phi_' num2str(sidelength)])
Phi = Phi(1:M,:);

%% Call the TFOCS solver
opts = [];
opts.restart    = 1000;
opts.maxIts     = 250;
x0 = Phi\y;
z0  = [];   % we don't have a good guess for the dual
W   = linop_TV( [sidelength,sidelength] );
normW           = linop_TV( [sidelength,sidelength], [], 'norm' );
opts.normW2     = normW^2;
mu  = 1.8e+2;
EPS = 1.0e-3;

tstart = cputime;
% [ x, out, optsOut ] = solver_sBPDN_W( A, W, b, EPS, mu, VEC(x0), z0, opts
% );
solver = @(mu,x0,z0,opts) solver_sBPDN_W( Phi, W, y, EPS, mu, x0, z0, opts);
[ x, out, optsOut ] = continuation(solver,5*mu,x0(:),z0,opts);
time_TFOCS = cputime - tstart;

% Reshape output; flip upside down
xR = reshape(x,sqrt(N),sqrt(N));
xR = flipud(xR);

% Normalize output
xR = (xR-min(min(xR)))/(max(max(xR))-min(min(xR)));

% Show reconstruction
figure(2);
imagesc(xR),colormap(gray);
text = ['TFOCS, CPU time: ',num2str(time_TFOCS),' sec.'];
title(text);
axis off;

avg_cpu_tfocs = avg_cpu_tfocs + time_TFOCS;

xR2 = xR;

%% pdNCG
optspdNCG.n   = sidelength^2;
optspdNCG.n1  = sidelength;
optspdNCG.n2  = sidelength;
optspdNCG.b   = y;
optspdNCG.A   = Phi;
optspdNCG.tau = 4.8899*2*norm(out.dual{2},inf)/norm(out.dual{1});
optspdNCG.mu = 1.0e-12;
optspdNCG.x0 = Phi\y;
optspdNCG.problem = 'tv';

tstart = cputime;
[xDC, dual, out] = pdNCGinterface(optspdNCG);
time_pdNCG = cputime - tstart;

% Reshape output; flip upside down
xR = reshape(xDC,sqrt(N),sqrt(N));
xR = flipud(xR);

% Normalize output
xR = (xR-min(min(xR)))/(max(max(xR))-min(min(xR)));

% Show reconstruction
figure(1);
imagesc(xR),colormap(gray);
text = ['pdNCG, CPU time: ',num2str(time_pdNCG),' sec.'];
title(text);
axis off;

avg_cpu_pdncg = avg_cpu_pdncg + time_pdNCG;

fprintf('Rellative difference for DICE: %d \n',norm(xR - xR2)/norm(xR2));

%cvx_begin 
%
%    variable xrec(sidelength,sidelength)
%    minimize optspdNCG.tau*norm(xrec(:,2:sidelength) - xrec(:,1:sidelength-1),1) + ...
%             optspdNCG.tau*norm(xrec(2:sidelength,:) - xrec(1:sidelength-1,:),1) + ...
%             sum_square(Phi*xrec(:) - y)
%cvx_end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% HERE FOR BALL %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test picture name. Options include 'Ball','Mug','Dice','Logo', 'R'
picname = 'Ball';

% Load measurements - Column Vector
eval(['load ' picname '_' num2str(sidelength)]);

% Image size
N = sidelength^2;

% Get desired number of samples
y = y(1:(round(N*subrate)));

% Number of measurements
M = length(y);

% Load measurement matrix and truncate
% Each row of Phi contains a measurement vector of length N
% and is a raster scan of the 0/1 pattern fed to the DMD
eval(['load Phi_' num2str(sidelength)])
Phi = Phi(1:M,:);

%% Call the TFOCS solver
opts = [];
opts.restart    = 1000;
opts.maxIts     = 250;
x0 = Phi\y;
z0  = [];   % we don't have a good guess for the dual
W   = linop_TV( [sidelength,sidelength] );
normW           = linop_TV( [sidelength,sidelength], [], 'norm' );
opts.normW2     = normW^2;
mu  = 1.8e+2;
EPS = 1.0e-3;

tstart = cputime;
% [ x, out, optsOut ] = solver_sBPDN_W( A, W, b, EPS, mu, VEC(x0), z0, opts
% );
solver = @(mu,x0,z0,opts) solver_sBPDN_W( Phi, W, y, EPS, mu, x0, z0, opts);
[ x, out, optsOut ] = continuation(solver,5*mu,x0(:),z0,opts);
time_TFOCS = cputime - tstart;

% Reshape output; flip upside down
xR = reshape(x,sqrt(N),sqrt(N));
xR = flipud(xR);

% Normalize output
xR = (xR-min(min(xR)))/(max(max(xR))-min(min(xR)));

% Show reconstruction
figure(4);
imagesc(xR),colormap(gray);
text = ['TFOCS, CPU time: ',num2str(time_TFOCS),' sec.'];
title(text);
axis off;

avg_cpu_tfocs = avg_cpu_tfocs + time_TFOCS;

xR2 = xR;

%% pdNCG
optspdNCG.n   = sidelength^2;
optspdNCG.n1  = sidelength;
optspdNCG.n2  = sidelength;
optspdNCG.b   = y;
optspdNCG.A   = Phi;
optspdNCG.tau = 5.5241*2*norm(out.dual{2},inf)/norm(out.dual{1}); % XXX explain this.
optspdNCG.mu = 1.0e-12;
optspdNCG.x0 = Phi\y;
optspdNCG.problem = 'tv';

tstart = cputime;
[xDC, dual, out] = pdNCGinterface(optspdNCG);
time_pdNCG = cputime - tstart;

% Reshape output; flip upside down
xR = reshape(xDC,sqrt(N),sqrt(N));
xR = flipud(xR);

% Normalize output
xR = (xR-min(min(xR)))/(max(max(xR))-min(min(xR)));

% Show reconstruction
figure(3);
imagesc(xR),colormap(gray);
text = ['pdNCG, CPU time: ',num2str(time_pdNCG),' sec.'];
title(text);
axis off;

avg_cpu_pdncg = avg_cpu_pdncg + time_pdNCG;

fprintf('Rellative difference for BALL: %d \n',norm(xR - xR2)/norm(xR2));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% HERE FOR Mug %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test picture name. Options include 'Ball','Mug','Dice','Logo', 'R'
picname = 'Mug';

% Load measurements - Column Vector
eval(['load ' picname '_' num2str(sidelength)]);

% Image size
N = sidelength^2;

% Get desired number of samples
y = y(1:(round(N*subrate)));

% Number of measurements
M = length(y);

% Load measurement matrix and truncate
% Each row of Phi contains a measurement vector of length N
% and is a raster scan of the 0/1 pattern fed to the DMD
eval(['load Phi_' num2str(sidelength)])
Phi = Phi(1:M,:);

%% Call the TFOCS solver
opts = [];
opts.restart    = 1000;
opts.maxIts     = 400;
x0 = Phi\y;
z0  = [];   % we don't have a good guess for the dual
W   = linop_TV( [sidelength,sidelength] );
normW           = linop_TV( [sidelength,sidelength], [], 'norm' );
opts.normW2     = normW^2;
mu  = 6.0e+1;
EPS = 1.0e-3;

tstart = cputime;
% [ x, out, optsOut ] = solver_sBPDN_W( A, W, b, EPS, mu, VEC(x0), z0, opts
% );
solver = @(mu,x0,z0,opts) solver_sBPDN_W( Phi, W, y, EPS, mu, x0, z0, opts);
[ x, out, optsOut ] = continuation(solver,5*mu,x0(:),z0,opts);
time_TFOCS = cputime - tstart;

% Reshape output; flip upside down
xR = reshape(x,sqrt(N),sqrt(N));
xR = flipud(xR);

% Normalize output
xR = (xR-min(min(xR)))/(max(max(xR))-min(min(xR)));

% Show reconstruction
figure(6);
imagesc(xR),colormap(gray);
text = ['TFOCS, CPU time: ',num2str(time_TFOCS),' sec.'];
title(text);
axis off;

avg_cpu_tfocs = avg_cpu_tfocs + time_TFOCS;

xR2 = xR;

%% pdNCG
optspdNCG.n   = sidelength^2;
optspdNCG.n1  = sidelength;
optspdNCG.n2  = sidelength;
optspdNCG.b   = y;
optspdNCG.A   = Phi;
optspdNCG.tau = 5.1877*2*norm(out.dual{2},inf)/norm(out.dual{1});
optspdNCG.mu = 1.0e-12;
optspdNCG.x0 = Phi\y;
optspdNCG.problem = 'tv';

tstart = cputime;
[xDC, dual, out] = pdNCGinterface(optspdNCG);
time_pdNCG = cputime - tstart;

% Reshape output; flip upside down
xR = reshape(xDC,sqrt(N),sqrt(N));
xR = flipud(xR);

% Normalize output
xR = (xR-min(min(xR)))/(max(max(xR))-min(min(xR)));

% Show reconstruction
figure(5);
imagesc(xR),colormap(gray);
text = ['pdNCG, CPU time: ',num2str(time_pdNCG),' sec.'];
title(text);
axis off;

avg_cpu_pdncg = avg_cpu_pdncg + time_pdNCG;

fprintf('Rellative difference for MUG: %d \n',norm(xR - xR2)/norm(xR2));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% HERE FOR R %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test picture name. Options include 'Ball','Mug','Dice','Logo', 'R'
picname = 'R';

% Load measurements - Column Vector
eval(['load ' picname '_' num2str(sidelength)]);

% Image size
N = sidelength^2;

% Get desired number of samples
y = y(1:(round(N*subrate)));

% Number of measurements
M = length(y);

% Load measurement matrix and truncate
% Each row of Phi contains a measurement vector of length N
% and is a raster scan of the 0/1 pattern fed to the DMD
eval(['load Phi_' num2str(sidelength)])
Phi = Phi(1:M,:);

%% Call the TFOCS solver
opts = [];
opts.restart    = 1000;
opts.maxIts     = 250;
x0 = Phi\y;
z0  = [];   % we don't have a good guess for the dual
W   = linop_TV( [sidelength,sidelength] );
normW           = linop_TV( [sidelength,sidelength], [], 'norm' );
opts.normW2     = normW^2;
mu  = 1.8e+1;
EPS = 1.0e-3;

tstart = cputime;
% [ x, out, optsOut ] = solver_sBPDN_W( A, W, b, EPS, mu, VEC(x0), z0, opts
% );
solver = @(mu,x0,z0,opts) solver_sBPDN_W( Phi, W, y, EPS, mu, x0, z0, opts);
[ x, out, optsOut ] = continuation(solver,5*mu,x0(:),z0,opts);
time_TFOCS = cputime - tstart;

% Reshape output; flip upside down
xR = reshape(x,sqrt(N),sqrt(N));
xR = flipud(xR);

% Normalize output
xR = (xR-min(min(xR)))/(max(max(xR))-min(min(xR)));

% Show reconstruction
figure(8);
imagesc(xR),colormap(gray);
text = ['TFOCS, CPU time: ',num2str(time_TFOCS),' sec.'];
title(text);
axis off;

avg_cpu_tfocs = avg_cpu_tfocs + time_TFOCS;

xR2 = xR;

%% pdNCG
optspdNCG.n   = sidelength^2;
optspdNCG.n1  = sidelength;
optspdNCG.n2  = sidelength;
optspdNCG.b   = y;
optspdNCG.A   = Phi;
optspdNCG.tau = 65*(2*norm(out.dual{2},inf)/norm(out.dual{1}));
optspdNCG.mu = 1.0e-12;
optspdNCG.x0 = Phi\y;
optspdNCG.problem = 'tv';

tstart = cputime;
[xDC, dual, out] = pdNCGinterface(optspdNCG);
time_pdNCG = cputime - tstart;

% Reshape output; flip upside down
xR = reshape(xDC,sqrt(N),sqrt(N));
xR = flipud(xR);

% Normalize output
xR = (xR-min(min(xR)))/(max(max(xR))-min(min(xR)));

% Show reconstruction
figure(7);
imagesc(xR),colormap(gray);
text = ['pdNCG, CPU time: ',num2str(time_pdNCG),' sec.'];
title(text);
axis off;

avg_cpu_pdncg = avg_cpu_pdncg + time_pdNCG;

fprintf('Rellative difference for R: %d \n',norm(xR - xR2)/norm(xR2));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% HERE FOR LOGO %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test picture name. Options include 'Ball','Mug','Dice','Logo', 'R'
picname = 'Logo';

% Load measurements - Column Vector
eval(['load ' picname '_' num2str(sidelength)]);

% Image size
N = sidelength^2;

% Get desired number of samples
y = y(1:(round(N*subrate)));

% Number of measurements
M = length(y);

% Load measurement matrix and truncate
% Each row of Phi contains a measurement vector of length N
% and is a raster scan of the 0/1 pattern fed to the DMD
eval(['load Phi_' num2str(sidelength)])
Phi = Phi(1:M,:);

%% Call the TFOCS solver
opts = [];
opts.restart    = 1000;
opts.maxIts     = 500;
x0 = Phi\y;
z0  = [];   % we don't have a good guess for the dual
W   = linop_TV( [sidelength,sidelength] );
normW           = linop_TV( [sidelength,sidelength], [], 'norm' );
opts.normW2     = normW^2;
mu  = 5.8e+2;
EPS = 1.0e-3;

tstart = cputime;
% [ x, out, optsOut ] = solver_sBPDN_W( A, W, b, EPS, mu, VEC(x0), z0, opts
% );
solver = @(mu,x0,z0,opts) solver_sBPDN_W( Phi, W, y, EPS, mu, x0, z0, opts);
[ x, out, optsOut ] = continuation(solver,5*mu,x0(:),z0,opts);
time_TFOCS = cputime - tstart;

% Reshape output; flip upside down
xR = reshape(x,sqrt(N),sqrt(N));
xR = flipud(xR);

% Normalize output
xR = (xR-min(min(xR)))/(max(max(xR))-min(min(xR)));

% Show reconstruction
figure(10);
imagesc(xR),colormap(gray);
text = ['TFOCS, CPU time: ',num2str(time_TFOCS),' sec.'];
title(text);
axis off;

avg_cpu_tfocs = avg_cpu_tfocs + time_TFOCS;

xR2 = xR;

%% pdNCG
optspdNCG.n   = sidelength^2;
optspdNCG.n1  = sidelength;
optspdNCG.n2  = sidelength;
optspdNCG.b   = y;
optspdNCG.A   = Phi;
optspdNCG.tau = 0.1376*2*norm(out.dual{2},inf)/norm(out.dual{1});
optspdNCG.mu = 1.0e-12;
optspdNCG.x0 = Phi\y;
optspdNCG.problem = 'tv';

tstart = cputime;
[xDC, dual, out] = pdNCGinterface(optspdNCG);
time_pdNCG = cputime - tstart;

% Reshape output; flip upside down
xR = reshape(xDC,sqrt(N),sqrt(N));
xR = flipud(xR);

% Normalize output
xR = (xR-min(min(xR)))/(max(max(xR))-min(min(xR)));

% Show reconstruction
figure(9);
imagesc(xR),colormap(gray);
text = ['pdNCG, CPU time: ',num2str(time_pdNCG),' sec.'];
title(text);
axis off;

avg_cpu_pdncg = avg_cpu_pdncg + time_pdNCG;

fprintf('Rellative difference for DICE: %d \n',norm(xR - xR2)/norm(xR2));

avg_cpu_pdncg = avg_cpu_pdncg/5;
avg_cpu_tfocs = avg_cpu_tfocs/5;

fprintf('Average CPU time for TFOCS: %d \n',avg_cpu_tfocs);
fprintf('Average CPU time for pdNCG: %d \n',avg_cpu_pdncg);

fprintf('Elapsed CPU time for this file: %d sec. \n', cputime - tt_file);
clear;

